/**
 *
 * This is the main PBotSerialSubController program file.
 *
 */

#include <stdio.h>
#include <propeller.h>
#include "simpletext.h"
#include "simpletools.h"
#include "pbotserialparser.h"
#include "sonarsensor.h"
#include "sonarsubsys.h"


enum SUB_CTL_PARAMS
{
    CMD_RX_PIN          = 25,
    CMD_TX_PIN          = 24,
    CMD_BAUD            = 115200,

    SONAR_CNT           = 2,
    SONAR_SWEEP_RATE    = 50,
    SONAR_TRIG_PULSE    = 20,
    SONAR_ECHO_TIMEOUT  = 25,
};


// Global storage for system objects
SonarSensor memSonars[SONAR_CNT];
SonarSubSys sonarSys(SONAR_SWEEP_RATE, SN_MODE_SINGLE_SWEEP);
PBotSerialParser cmdParser(CMD_RX_PIN, CMD_TX_PIN, CMD_BAUD);

int  main(void)
{
    int result = 0;
    int validate = 0;

    // Configure Sonar Subsystem
    putStr("Building Sonar Subsystem:\n");
    putStr("--Initialize Sonar Sensors\n");
    memSonars[0].init(8,9,SONAR_TRIG_PULSE,SONAR_ECHO_TIMEOUT);
    memSonars[1].init(10,11,SONAR_TRIG_PULSE,SONAR_ECHO_TIMEOUT);
    putStr("--Adding Sensors to Sonar Subsystem: \n");
    for (int i=0; i<SONAR_CNT; i++)
    {
        validate = sonarSys.addSensor(memSonars[i]);
        putStr("  --[");
        putDec(i);
        putStr("] ");

        if(validate < 0)
        {
            putStr("Failed to add sensor! Aborting. \n");
            return -1;
        }

        putStr("Added Successfully\n");
    }

    // Register the sonar subsystem with the command parser
    if(!cmdParser.isReady())
    {
        putStr("Command Parser Not Initialized! Aborting.\n");
        return -1;
    }

    if(cmdParser.addSubSys(sonarSys, "SN") < 0)
    {
        putStr("Command Parser failed to add SubSystem! Aborting.\n");
        return -1;
    }

    putStr("Starting Command Loop\n");

    while(1)
    {
        result = cmdParser.checkCmd();
        if (result > 0)
        {
            int tIndex = 0;
            String term;

            putStr("Command Detected:  [");
            putStr(cmdParser.getCmd());
            putStr("]\n");
        }
    }

    return 0;
}

