#ifndef PBOTSERIALPARSER_H
#define PBOTSERIALPARSER_H

#include "fdserial.h"
#include "pstring.h"

#define CMD_START   '['
#define CMD_END     ']'
#define TERM_DELIM  ':'
#define ACK_CHAR    'A'
#define ERR_CHAR    'E'

enum CMDP_PARAMS
{
    MAX_CMD_SIZE            = 30,
    MAX_TERMS               = 10,
    MAX_SUBSYS              = 16
};

enum CMDP_RETURNS
{
    CMD_PARSER_OK           = 0,
    CMD_PARSER_RESULT       = 1,
    CMD_PARSER_ERR_GEN      = -1,
    CMD_PARSER_ERR_BAD_VAL  = -2,
    CMD_PARSER_ERR_BAD_TERM = -3,
    CMD_PARSER_ERR_BAD_CMD  = -4,
    CMD_PARSER_NO_VAL       = -255,
    CMD_OK                  = 1,
};

/*
    Class to interface with commanding host over serial TX/RX wires.  This class manages the communication
    with the host.  Stores the incomming command, and packages any return values.  Interfaces with various
    subsystems to inform them of incomming commands and accept return values.
*/
class ISubSys;

class PBotSerialParser
{
public:
    PBotSerialParser();
    PBotSerialParser(int rxPin, int txPin, int b_rate);
    ~PBotSerialParser() {};
    int init(int rxPin, int txPin, int b_rate); // Initialize the command parsing object

    int isReady();                                      // Indicates if the object is properly initialized.
    int checkCmd();                                     // Checks to see if a command is incoming.
    char* getCmd();                                     // Returns the cmd string minus the enclosing "[]"
    int getTerm(String& term, int termIndex);           // Returns the term requested from the command string.
    int getTermCnt();                                   // Returns the number of terms in the current command.
    int sendResp();                                     // Sends a subsystem's response string back to host.
    int addSubSys(ISubSys& subSys, const char* name);

protected:
    int dispCmd();                                      // Dispatch the command to the proper subsystem.
    void clrTerms();                                    // Clear the terms from the last command
    void clrSubSys();                                   // Clears the arrays for storing subsystems


protected:
    int         ready;                      // Indicates the object is initialized and ready to operate
    fdserial    *p_cmdTerm;                 // Serial Terminal to use when
    String      rawCommand;                 // Last valid command obtained.
    int         termCnt;                    // Number of terms in the current command
    int         terms[MAX_TERMS];           // Postion of term delimiters.
    String      subSysName[MAX_SUBSYS];     // String array that contains the term to activate a given subsystem
    ISubSys*    p_subSys[MAX_SUBSYS];       // Arrary of pointers to subsystems
};


#endif // PBOTSERIALPARSER_H
