#include "simpletools.h"
#include "simpletext.h"
#include "isonarsensor.h"
#include "sonarsubsys.h"


SonarSubSys::SonarSubSys()
{
    p_parser = 0;
    ready = 0;
    mbox.mode = -1;
    mbox.rate = -1;
    units = 0;
    init(50, SN_MODE_SINGLE_SWEEP, SN_IN);
}

/** @brief (one liner)
  *
  * (documentation goes here)
  */
SonarSubSys::SonarSubSys(int r, int m, int u)
{
    p_parser = 0;
    ready = 0;
    mbox.mode = -1;
    mbox.rate = -1;
    units = 0;
    init(r, m, u);
}


/** @brief (one liner)
  *
  * (documentation goes here)
  */
SonarSubSys::SonarSubSys(PBotSerialParser& parser, int r, int m, int u)
{
    p_parser = 0;
    ready = 0;
    mbox.mode = -1;
    mbox.rate = -1;
    units = 0;
    init(parser, r, m, u);
}


/** @brief (one liner)
  *
  * (documentation goes here)
  */
void SonarSubSys::init(int r, int m, int u)
{
    setRate(r);
    setMode(m);
    setUnits(u);
    clrArrays();
}


/** @brief Initialize the Subsystem data.
  *
  * (documentation goes here)
  */
void SonarSubSys::init(PBotSerialParser& parser, int r, int m, int u)
{
    p_parser = &parser;
    setRate(r);
    setMode(m);
    setUnits(u);
    clrArrays();
    ready = 1;
    strReturn = "";
}


/** @brief Indicates if the object is fully initialized and ready to use
  *
  * (documentation goes here)
  */
int SonarSubSys::isReady()
{
    if (ready == 0)
    {
        if (p_parser)
        {
            ready = 1;
        }
    }

    return ready;
}


/** @brief Fetches and executes the command the parser has indicated is ready.
  *
  * (documentation goes here)
  */
int SonarSubSys::execCmd()
{
    strReturn = "";
    String term;
    if(p_parser->getTerm(term, 1) < 0)
        return CMD_PARSER_ERR_BAD_CMD;

    if(term == SN_ACT_RD)
    {
        if(p_parser->getTerm(term, 2) < 0)
        return CMD_PARSER_ERR_BAD_CMD;

        if(term == SN_TARG_SENSOR)
        {
            int termVal = 0;
            int result = 0;
            if (p_parser->getTerm(term, 3) < 0)
                return CMD_PARSER_ERR_BAD_CMD;

            if(term == SN_ALL_SENSORS)
            {
                return readAllSensors();
            }

            termVal = term.toInt();
            if(termVal >= mbox.cnt or termVal < 0)
                return CMD_PARSER_ERR_BAD_VAL;

            result = readSensor(termVal);
            if(result >= 0)
            {
                strReturn = String(result);
                return CMD_PARSER_RESULT;
            }

        }

        return 1;
    }
    
    if(term == SN_ACT_WR)
    {
        int result;

        if(p_parser->getTerm(term, 2) < 0)
            return CMD_PARSER_ERR_BAD_CMD;

        if(term == SN_TARG_SCAN_RATE)
        {
            if (p_parser->getTerm(term, 3) < 0)
                return CMD_PARSER_ERR_BAD_CMD;

            result = writeScanRate(term);
            strReturn = String(result);
            return CMD_PARSER_RESULT;
        }

        if(term == SN_TARG_UNITS)
        {
            if (p_parser->getTerm(term, 3) < 0)
                return CMD_PARSER_ERR_BAD_CMD;

            result = writeUnits(term);
            strReturn = String(result);
            return CMD_PARSER_RESULT;
        }

        if(term == SN_TARG_MODE)
        {
            if (p_parser->getTerm(term, 3) < 0)
                return CMD_PARSER_ERR_BAD_CMD;

            result = writeMode(term);
            strReturn = String(result);
            return CMD_PARSER_RESULT;
        }

        if(term == SN_TARG_SCANPAT)
        {
            if(p_parser->getTerm(term, 3) < 0)
                return CMD_PARSER_ERR_BAD_CMD;

            result = writeScanPattern(term);
            strReturn = String(result);
            return CMD_PARSER_RESULT;
        }

        return CMD_PARSER_ERR_BAD_CMD;
    }
    else
    {
        return CMD_PARSER_ERR_BAD_CMD;
    }

    return 0;
}


/** @brief Add a sonar sensor to the subsystem. Sensor can be any that implements ISonarSensor
  *
  * (documentation goes here)
  */
int SonarSubSys::addSensor(ISonarSensor& sensor)
{
    int i = -1;
    int flag = 1;

    while(flag)
    {
        i++;
        if(i > SN_MAX_SENSORS)
        {
            return CMD_PARSER_ERR_BAD_VAL;
        }

        if(mbox.p_sensorArry[i] == 0)
        {
            mbox.p_sensorArry[i] = &sensor;
            flag = 0;
            mbox.cnt++;
        }
    }

    return i;
}


/** @brief Remove the sensor at the given sensor index.
  *
  * (documentation goes here)
  */
int SonarSubSys::removeSensor(int i)
{
    if(mbox.p_sensorArry[i])
    {
        mbox.p_sensorArry[i] = 0;
        mbox.cnt--;
        return i;
    }

    return CMD_PARSER_ERR_BAD_VAL;
}



/** @brief Read the sensor at the given sensor index
  *
  * @param int i: Index to the sensor to read.
  * @return int: Result in the units this object is set to operate in.  Inch results are in tenths
  *         of an inch.  Centimeter results are in whole centimeters.
  */
int SonarSubSys::readSensor(int i)
{
    int result = -1;
    int ms_tof = 0;

    if (i > SN_MAX_SENSORS)
        return CMD_PARSER_ERR_BAD_VAL;

    if (!mbox.p_sensorArry[i])
        return CMD_PARSER_ERR_GEN;

    if (units == SN_IN)
    {
        ms_tof = (mbox.p_sensorArry[i]->trigger() / 1000);
        if (ms_tof < 0)
            return CMD_PARSER_NO_VAL;

        result = mbox.p_sensorArry[i]->getRange_in();
    }
    else
    {
        ms_tof = (mbox.p_sensorArry[i]->trigger() / 1000);
        if (ms_tof < 0)
            return CMD_PARSER_NO_VAL;

        result = mbox.p_sensorArry[i]->getRange_cm();
    }

    pause(mbox.rate - ms_tof);
    return result;
}


int SonarSubSys::readAllSensors()
{
    int result = 0;
    for(int i=0; i<mbox.cnt; i++)
    {
        result = readSensor(i);
        strReturn += result;

        if (i < (mbox.cnt - 1))
            strReturn += ":";
    }

    return 1;
}


/** @brief Read the currently set scan rate in millisecs
  *
  * (documentation goes here)
  */
int SonarSubSys::readScanRate()
{
    return mbox.rate;
}


/** @brief Read the currently set scan mode
  *
  * (documentation goes here)
  */
int SonarSubSys::readMode()
{
    return mbox.mode;
}


/** @brief Set the scan rate in millisecs
  *
  * (documentation goes here)
  */
int SonarSubSys::writeScanRate(const String& term)
{
    int val = term.toInt();
    return setRate(val);
}


/** @brief Set the scan rate in millisecs
  *
  * (documentation goes here)
  */
int SonarSubSys::writeScanPattern(const String& term)
{
    const int val = term.toInt();

    if (val > 255 or val < 0)
        return CMD_PARSER_ERR_BAD_VAL;

    int testBit = 0;
    for (int i=0; i<mbox.cnt; i++)
    {
        testBit = val & (1<<i);
        if(testBit)
            mbox.p_sensorArry[i]->enable(1);
        else
            mbox.p_sensorArry[i]->enable(0);
    }

    return val;
}


/** @brief Set the current scan mode
  *
  * (documentation goes here)
  */
int SonarSubSys::writeMode(const String& term)
{
    int val = term.toInt();
    return setMode(val);
}


/** @brief Set the current scan mode
  *
  * (documentation goes here)
  */
int SonarSubSys::writeUnits(const String& term)
{
    int val = term.toInt();
    return setUnits(val);
}


/*  Clear all array data in this object */
int SonarSubSys::clrArrays()
{
    for(int i=0; i < SN_MAX_SENSORS; i++)
    {
        mbox.p_sensorArry[i] = 0;
    }
    mbox.cnt = 0;
}


/*  Validate and set the rate variable */
int SonarSubSys::setRate(int r)
{
    if (r == 0)
    {
        mbox.rate = 0;
    }
    else if (r >= 50 )
    {
        mbox.rate = r;
    }
    else
    {
        mbox.rate = 50;
    }

    return mbox.rate;
}


/* Validate and set the mode variable */
int SonarSubSys::setMode(int m)
{
    if (m > -1 and m < 2)
    {
        mbox.mode = m;
    }
    else
    {
        mbox.mode = 0;
    }

    return mbox.mode;
}


/* Validate and set the units varaible */
int SonarSubSys::setUnits(int u)
{
    if (u)
        units = SN_CM;
    else
        units = SN_IN;

    return units;
}
